%-------------------------------------------------------------------------%
%This program estimates the switching costs for the non normalized methods 
%(N-N)for the period 1980-1990. 
%-------------------------------------------------------------------------%
clear

load ryan
load coef
load after % use after/before for (before/after) logit coefficients
load boot_nonparam_after % use after/before for (before/after) bootstrap of ccps

boots=boot_after;

%% Cournot Game (per period profit)

% Following Ryan(2013) 
% Cost estimations by Ryan(2013)
mc=31.58;
cpc=1.239;
cptshold=1.916;
mc90=2.41;
cpc90=-.0299;
cptshold90=.0917;
mc=mc + mc90;
cpc=cpc + cpc90;
cptshold=cptshold + cptshold90;

% Demand parameters by Ryan, specification III table 3
alfa0=20.38;
alfa1=-2.96;

d(1)=exp(alfa0);
d(2)=1/alfa1;

c(1)=mc;
c(2)=cpc;

th=exp(cptshold)/(1+exp(cptshold));

%---------------------------Creating Categories---------------------------%

% number of Categories
capy=cap;
capy(capy==0)=[];


a=prctile(capy,per);

cat=zeros(size(cap,1),1);

for i=2:size(per,2)   
       cat(a(i-1)<=cap &  a(i)>cap )=i-1;       
end

cat(cap>=a(size(per,2)))=size(per,2);

options = optimset('Display','off','GradObj','off','MaxIter',80000,'TolX',1e-10,'TolFun',1e-10);

%-------------------Players, actions and profit factor--------------------%

p=4; %players
ac=categories; %actions
fact=1000000000; %Factor dividing the profits

%--------------------Computing the states---------------------------------%
[s0, s]=state(ac,p);

pro=zeros(size(s,1),1);

a=[a(2:end) max(capy)]; %capacities trasholds

%-------------------- Calculate Instantaneous Variable Profits -----------%

for i=1:size(s,1) % Loop over the states observed by a representative firm
    
    if s(i,1)>0
       
        qi=ones(sum(s(i,3:end))+1,1);
        lim=[a(s(i,1));ones(s(i,3),1)*a(1);ones(s(i,4),1)*a(2);ones(s(i,5),1)*a(3);ones(s(i,6),1)*a(4);ones(s(i,7),1)*a(5)];
        lb=0*lim;
        ub=lim;

        [qu, ~]=lsqnonlin(@(q) profit(q,d,c,th,lim),qi,lb,ub, options);
        
        if (qu(1)/lim(1))>th      
           mc= c(1);
        else
           mc=c(1)+(lim(1)-qu(1))^(2)*c(2); 
        end
        
        pro(i)=((sum(qu)/d(1))^d(2)-mc)*qu(1);

    end
    
end
pro=pro/fact; %Profit Units


%% Estimate Sunk Costs
dif=8;
siz=size(s,1);
ac=ac-1;

kk=1;
ii=1;
for i=1:(size(boots,2)/ac)
    coef_boot(ii:ii+dif-1,:)=boots(:,kk:kk+5-1);
    kk=kk+5;
    ii=ii+dif;
end

coef=coef(1:dif,:);
coef=[coef;coef_boot];

[id]=dum2(s(:,1)); % Calculate matrix of SC dummies

aux=[];
ii=1;
for i=1:ac
    
    aux1=id(ii:ii+size(s,1)-1,:);
    aux1(1,:)=[];
    aux1(:,1:5)=[];
    aux=[aux;aux1];
    ii=ii+size(s,1);
end

id=aux;

s0=s0(s0(:,1)==0,:);

rep=(size(boots,2)/ac)+1; %number of bootstrap repetitions +1 
s_costs =zeros(rep,25); % SCs
    
a_a=[[(1:4)' (2:5)'];[1 3;2 4;3 5];[1 4;2 5];[1 5]];

a_aux=kron((1:5)',ones(size(s,1),1));
sunk_a=zeros(rep,size(a_a,1));

l=1;
for j=1:rep   %Loop over number of bootstrapsimulations
    
    coe=coef(l:l+dif-1,:);
    
    prob_act=probs(coe,s);
    prob_act(prob_act<=1e-15)=1e-15;
    prob_act=prob_act./repmat(sum(prob_act,2),1,size(prob_act,2));    
    pax=inverso(prob_act);
                  
    for jj=1:size(a_a,1)
            
        v1=pax(a_aux==a_a(jj,1),:);
        v2=pax(a_aux==a_a(jj,2),:);
        sunk_a(j,jj)= mean(v1(s(:,1)==a_a(jj,1)) + v2(s(:,1)==a_a(jj,2)) - v1(s(:,1)==a_a(jj,2)) - v2(s(:,1)==a_a(jj,1)));
            
     end

     z1=transition(s,s0,prob_act);
     z=z1-repmat(z1(1,:),size(z1,1),1);
     z(1,:)=[];
    
     t=kron(prob_act,ones(1,size(z1,2))).*repmat(z1,1,ac+1);  
        
     mz= eye(size(z,1))-z/(z'*z)*z';
     aux=[];
     for i=1:ac
         aux=blkdiag(aux,mz);
     end
     mz=aux;
     
     paux=[];
     ii=1;
     for i=1:ac
    
        aux1=pax(ii:ii+size(s,1)-1,:);
        aux1=aux1-repmat(aux1(1,:),size(aux1,1),1);
        aux1(1,:)=[];
    
        paux=[paux;aux1];
        ii=ii+size(s,1);
      end
      pax=paux;
       
      y=mz*pax;
      x=mz*id;
      s_costs(j,:) = ((x'*x)\(x'*y))';
       
      l=l+dif;        

end

s_costs(:,1:20)=[];
s_costs=[s_costs sunk_a];

% Eliminating possible Not a Numbers
s_costs(isnan(sum(s_costs,2)),:)=[];

save('after_sunkallsunks','s_costs')

clear