%-------------------------------------------------------------------------%
%This program estimates the parameters, and standard deviations presented 
%in tables 5 and 7 of "Joint Analysis of the Discount Factor and 
%Payoff Parameters in Dynamic Discrete Choice Games," Fabio A. Miessi 
%Sanches, Daniel Silva Junior, Sorawoot Srisuma, and Tatiana Komarova
%-------------------------------------------------------------------------%

clear
load ryan %Obtained directly from raw data, contains information about firms, produciton and location 
load coef %Coefficients for the CCPs
load after % use after/before for (before/after) logit coefficients
load boot_nonparam_after % use after/before for (before/after) bootstrap of ccps

boots=boot_after;

%% Cournot Game (per period profit)

% Following Ryan(2013) 
% Cost estimations by Ryan(2013)
mc=31.58;
cpc=1.239;
cptshold=1.916;
mc90=2.41;% Aditional costs after 1990
cpc90=-.0299;% Aditional costs after 1990
cptshold90=.0917;% Aditional costs after 1990
mc=mc + mc90;
cpc=cpc + cpc90;
cptshold=cptshold + cptshold90;

% Demand parameters by Ryan(2013), specification III table 3
alfa0=20.38;
alfa1=-2.96;

d(1)=exp(alfa0);
d(2)=1/alfa1;

c(1)=mc;
c(2)=cpc;

th=exp(cptshold)/(1+exp(cptshold));

%---------------------------Creating Categories---------------------------%

% number of Categories
capy=cap;
capy(capy==0)=[];

a=prctile(capy,per);

cat=zeros(size(cap,1),1);

for i=2:size(per,2)
           
    cat(a(i-1)<=cap &  a(i)>cap )=i-1; 
        
end
    
cat(cap>=a(size(per,2)))=size(per,2);

options = optimset('Display','off','GradObj','off','MaxIter',100000000000,'TolX',1e-20,'TolFun',1e-20);

%-------------------Players, actions and profit factor--------------------%

p=4; %players
ac=categories; %actions
fact=1000000; %Factor dividing the profits

%--------------------Computing the states---------------------------------%

[s0, s]=state(ac,p);

pro=zeros(size(s,1),1);

a=[a(2:end) max(capy)];%capacities trasholds

%-------------------- Calculate Instantaneous Variable Profits -----------%

for i=1:size(s,1)% Loop over the states observed by a representative firm
    
    if s(i,1)>0
       
        qi=ones(sum(s(i,3:end))+1,1);
        lim=[a(s(i,1));ones(s(i,3),1)*a(1);ones(s(i,4),1)*a(2);ones(s(i,5),1)*a(3);ones(s(i,6),1)*a(4);ones(s(i,7),1)*a(5)];
        
        lb=0*lim;
        ub=lim;

        [qu, ~]=lsqnonlin(@(q) profit(q,d,c,th,lim),qi,lb,ub, options);% quantity in equilibrium in a Cournot game with capacity constraint
        
        if (qu(1)/lim(1))>th     
           mc= c(1);   
        else
           mc=c(1)+(lim(1)-qu(1))^(2)*c(2); 
        end
        
        pro(i)=((sum(qu)/d(1))^d(2)-mc)*qu(1);% per period profits

    end
end
pro=pro/fact; %Profit Units

%% Estimate Switching Costs/Discount Rate 
dif=8;
siz=size(s,1);
ac=ac-1;

kk=1;
ii=1;
for i=1:(size(boots,2)/ac)
    coef_boot(ii:ii+dif-1,:)=boots(:,kk:kk+5-1);
    kk=kk+5;
    ii=ii+dif;
end

coef=coef(1:dif,:);
coef=[coef;coef_boot];

[id, a]=dum(s(:,1)); % Calculate matrix of SC dummies

s0=s0(s0(:,1)==0,:);

rep=(size(boots,2)/ac)+1; %number of bootstrap repetitions +1 
scs_prof =zeros(rep,26); % [OP_costs SCs(25)]
scs_sss2a=zeros(rep,26); % [OP_costs SCs(25)]
discfactor_prof =zeros(rep,1);
discfactor_sss2a=zeros(rep,1);

options = optimset('Display','none','GradObj','off','MaxIter',80000,'TolX',1e-10,'TolFun',1e-10);


l=1;
for j=1:rep   %Loop over number of bootstrapsimulations

    coe=coef(l:l+dif-1,:);
    
    prob_act=probs(coe,s);
    prob_act(prob_act<=1e-15)=1e-15;
    prob_act=prob_act./repmat(sum(prob_act,2),1,size(prob_act,2));
    
    z=transition(s,s0,prob_act);
    t=kron(prob_act,ones(1,size(z,2))).*repmat(z,1,ac+1);
    
    
    %------------Method A(Estimation + bootstrap)-------------------------%
    try
    
    mz= eye(size(s,1))-z/(z'*z)*z';
    aux=[];
    for i=1:ac
        aux=blkdiag(aux,mz);
    end
    mz=aux;
    y=mz*inverso(prob_act);
    x=mz*id;
    scs_sss2a(j,2:end) = ((x'*x)\(x'*y))';
    
    estimation=1; %index for beta, method A
    [ss2a_aux]=fmincon(@(x) gmm(s,pro,prob_act,ac,x,t,scs_sss2a(j,2:end),estimation),0.9,[],[],[],[],0.0,0.999,[],options);
    discfactor_sss2a(j,1)=ss2a_aux;
    estimation=2; %index for opcosts
    [ss2a_aux]=gmm(s,pro,prob_act,ac,discfactor_sss2a(j,1),t,scs_sss2a(j,2:end),estimation);
    scs_sss2a(j,1)=ss2a_aux;

    %------------Method B(Estimation + bootstrap)-------------------------%

    % results=[SCs Beta OP_costs]
    estimation=3; %index for beta, method B
    [discfactor_prof(j)]=fmincon(@(x) gmm(s,pro,prob_act,ac,x,t,ones(size(scs_sss2a(j,:))),estimation),0.9,[],[],[],[],0.0,0.999,[],options);
    estimation=4;%index for operational costs and s costs, method B 
    [scs_prof(j,:)]=gmm(s,pro,prob_act,ac,discfactor_prof(j,1),t,ones(size(scs_sss2a(j,:))),estimation)';
    
    catch
    end
    
    l=l+dif;
    
end

%-------------------------- FINAL TABLES ---------------------------------%
% Eliminating possible Not a Numbers
a=isnan(sum(scs_prof,2));
scs_prof(a==1,:)=[];
scs_sss2a(a==1,:)=[];

a=isnan(sum(scs_sss2a,2));
scs_prof(a==1,:)=[];
scs_sss2a(a==1,:)=[];



W_prof= scs_prof(2:end,2:end);


%----------------------Joint Test-----------------------------------------%
% variance covariance matrix SSS2a test SCS
W_sss2a=scs_sss2a(2:end,2:end);
W_diff=W_prof-W_sss2a;
V_diff=(W_diff-repmat(mean(W_diff,1),size(W_diff,1),1))';
V_diff=(V_diff*V_diff')/(size(W_diff,1));

coef_diff=scs_prof(1,2:end)-scs_sss2a(1,2:end);
Q_diff_after=coef_diff*inv(V_diff)*coef_diff';

pv=1-chi2cdf(Q_diff_after,25);



t_entriesA=[scs_prof(1,2:end);std(scs_prof(2:end,2:end))];
t_entriesB=[scs_sss2a(1,2:end);std(scs_sss2a(2:end,2:end))];

t1=zeros(10,6);
t2=zeros(10,6);


t1(:,1)=[t_entriesA(:,1);t_entriesA(:,2);t_entriesA(:,3);t_entriesA(:,4);t_entriesA(:,5)];
t1(3:end,2)=[t_entriesA(:,6);t_entriesA(:,7);t_entriesA(:,8);t_entriesA(:,9)];
t1(:,3)=[t_entriesA(:,10);0;0;t_entriesA(:,11);t_entriesA(:,12);t_entriesA(:,13)];
t1(:,4)=[t_entriesA(:,14);t_entriesA(:,15);0;0;t_entriesA(:,16);t_entriesA(:,17)];
t1(:,5)=[t_entriesA(:,18);t_entriesA(:,19);t_entriesA(:,20);0;0;t_entriesA(:,21)];
t1(:,6)=[t_entriesA(:,22);t_entriesA(:,23);t_entriesA(:,24);t_entriesA(:,25);0;0];

t2(:,1)=[t_entriesB(:,1);t_entriesB(:,2);t_entriesB(:,3);t_entriesB(:,4);t_entriesB(:,5)];
t2(3:end,2)=[t_entriesB(:,6);t_entriesB(:,7);t_entriesB(:,8);t_entriesB(:,9)];
t2(:,3)=[t_entriesB(:,10);0;0;t_entriesB(:,11);t_entriesB(:,12);t_entriesB(:,13)];
t2(:,4)=[t_entriesB(:,14);t_entriesB(:,15);0;0;t_entriesB(:,16);t_entriesB(:,17)];
t2(:,5)=[t_entriesB(:,18);t_entriesB(:,19);t_entriesB(:,20);0;0;t_entriesB(:,21)];
t2(:,6)=[t_entriesB(:,22);t_entriesB(:,23);t_entriesB(:,24);t_entriesB(:,25);0;0];

%Tables: Numerical values
tb=[t1;zeros(1,6);t2];
tb(end+1:end+2,1)=[Q_diff_after;pv];%table 5
 
tb2A=[discfactor_prof(1) scs_prof(1,1);std(discfactor_prof(2:end)) std(scs_prof(2:end,1))];
tb2B=[discfactor_sss2a(1) scs_sss2a(1,1);std(discfactor_sss2a(2:end)) std(scs_sss2a(2:end,1))];
tb2=[tb2A;0 0;tb2B];%table 7

%Tables: String format
t1s=num2str(round(t1,4));
t1s=[['ait1=1 ';'       ';'ait1=2 ';'       ';'ait1=3 ';'       ';'ait1=4 ';'       ';'ait1=5 ';'       '] t1s];
t1s=[['         ait-1=0     ait-1=1      ait-1=2      ait-1=3      ait-1=4      ait-1=5'];t1s];
t1s=[['Method A                                                                        '];t1s];

t2s=num2str(round(t2,4));
t2s=[['ait1=1 ';'       ';'ait1=2 ';'       ';'ait1=3 ';'       ';'ait1=4 ';'       ';'ait1=5 ';'       '] t2s];
t2s=[['         ait-1=0     ait-1=1      ait-1=2      ait-1=3      ait-1=4      ait-1=5'];t2s];
t2s=[['Method B                                                                        '];t2s];

tbn=[t1s;t2s];
test=num2str(round([Q_diff_after;pv],4));
test1=['Statistic  ' num2str(round(Q_diff_after,4)) ];
test2=['p-value    ' num2str(round(pv,4)) ];

tbn=[tbn;'Specification Test' blanks(62);test1 blanks(62);test2 blanks(64)];%table 5 


tbn2aA=['Discount Factor '; [num2str(round(discfactor_prof(1),4)) blanks(11)]; [num2str(std(discfactor_prof(2:end)),4) blanks(10)]];
tbn2bA=['Operating Cost '; [num2str(round(scs_prof(1,1),4)) blanks(8)]; [num2str(std(scs_prof(2:end,1)),4) blanks(10)]];
tbn2A=[tbn2aA tbn2bA];

tbn2aB=['Discount Factor '; [num2str(round(discfactor_sss2a(1),4)) blanks(10)]; [num2str(std(discfactor_sss2a(2:end)),4) blanks(10)]];
tbn2bB=['Operating Cost '; [num2str(round(scs_sss2a(1,1),4)) blanks(8)]; [num2str(std(scs_sss2a(2:end,1)),4) blanks(9)]];
tbn2B=[tbn2aB tbn2bB];

tbn2=['Method A' blanks(23);tbn2A;'Method B' blanks(23);tbn2B];%table 7

save('results_after','scs_prof','scs_sss2a') %It will be used for the final table

clearvars -except tb tbn tb2 tbn2