%-------------------------------------------------------------------------%
%This program Computes the equilibrium probabilities for the estimates presented 
%in tables 1 to 3 of "Joint Analysis of the Discount Factor and Payoff Parameters 
%in Dynamic Discrete Choice Games," Fabio A. Miessi Sanches, Daniel Silva 
%Junior, Sorawoot Srisuma, and Tatiana Komarova.
%-------------------------------------------------------------------------%

clear;

%-----------------------------Parameters----------------------------------%
%Discont factor
beta=.9;

%States
s = [0 0;0 1;1 0;1 1];

%Initial paramters
pie=[-.2 1.2 -1.2];

%Initial guess 
lb=zeros(size(s,1),2);
ub=ones(size(s,1),2);
options = optimset('Display','iter','GradObj','off','MaxIter',8000,'TolX',1e-15,'TolFun',1e-15);

%--------------------- Estimation Equilibrium 1---------------------------%

%probability of choosing a=1
prob1= [0.73,0.28;0.61,0.42;0.80,0.22;0.75,0.29]; %Initial Values
[pi1,~]=lsqnonlin(@(prob) gmm(pie,prob ,s,beta),prob1,lb,ub, options);
save('eq1','pi1')

%--------------------- Estimation Equilibrium 2---------------------------%

%probability of choosing a=1
prob1=[0.62,0.53;0.31,0.84;0.83,0.30;0.61,0.58]; %Initial Values
[pi1,~]=lsqnonlin(@(prob) gmm(pie,prob ,s,beta),prob1,lb,ub, options);
save('eq2','pi1')

%--------------------- Estimation Equilibrium 3---------------------------%

%probability of choosing a=1
prob1=[0.58,0.58;0.30,0.84;0.84,0.30;0.59,0.59]; %Initial Values
[pi1,~]=lsqnonlin(@(prob) gmm(pie,prob ,s,beta),prob1,lb,ub, options);
save('eq3','pi1')

%------------------------------Outputs------------------------------------%
%Outputs:
%       eq1; eq2;eq3. The files contain equilibrium probabilities for each 
%equilibrium used for the estimations
%-------------------------------------------------------------------------%