%-------------------------------------------------------------------------%
%This program estimates the parameters, standard deviation and mean square 
%errros presented in table 3 of "Joint Analysis of the Discount Factor and 
%Payoff Parameters in Dynamic Discrete Choice Games," Fabio A. Miessi 
%Sanches, Daniel Silva Junior, Sorawoot Srisuma, and Tatiana Komarova
%-------------------------------------------------------------------------%

clear

%States
s = [0 0;0 1;1 0;1 1];
sc=[0;1;1]; %Used to estimate N-N

%Scrap values
wr=[0 .1 .2 ]; %Scrap values used for the simulation
options = optimset('Display','Off','GradObj','off','MaxIter',8000,'TolX',1e-10,'TolFun',1e-10, 'MaxFunEvals', 1000);


%% Sample size: 100
load eq3

jj=1;

probability=[pi1(:,1);pi1(:,2)];
n=[10000 100]; %[number of repetitions   sample size]

rng(390); %seed

[prob, ~]=sim_crosssection(n,probability); %simulate probabilities

parsF=zeros(size(prob,1)*4,size(wr,2));
pars_beta=zeros(size(prob,1)*4,size(wr,2));
par_NN=zeros(size(prob,1)*4,1);

k=1;
l=1;

for i=1:size(prob,1) %Loop over the repetions 
    
    probs=[prob(i,1:4)' prob(i,5:8)'];
 
    for j=1:size(wr,2) %Loop over the scrap values
        
        try
           
           %Method B 
           zf=-[1;1;0;0];
           zw=[0;0;1;1;];
           z1=[probs(:,2) 1-probs(:,2)];
           x=[z1 zf];
           zwp=norminv(1-probs(:,1))-zw*wr(j);
           aux=((x'*x)\(x'*zwp))';
           parsF(l+1,j)=aux(3);  
           parsF(l,j) = lsqnonlin(@(dis) gmm_estimationF(probs,s,dis,wr(j),1,parsF(l+1,j)),.9,0,1-0.000001, options);
           parsF(l+2:l+3,j) = gmm_estimationF(probs,s,parsF(l,j),wr(j),0,parsF(l+1,j)); 
            
           %Method A 
           pars_beta(l,j)=lsqnonlin(@(dis) gmm_estimation(probs,s,dis,wr(j),1),.9,0,1-0.000001, options);
           pars_beta(l+1:l+3,j)=gmm_estimation(probs,s,pars_beta(l,j),wr(j),0);           


        catch
        end   
         

    end
    
    %N-N
    z1=[1-probs(:,2) probs(:,2)];
    z1(2:end,1)= z1(2:end,1)-z1(1,1);
    z1(2:end,2)= z1(2:end,2)-z1(1,2);
    z1(1,:)=[];
    z1=z1(:,1);
    x=[z1 sc];
    zwp=norminv(probs(:,1));
    zwp(2:end)=zwp(2:end)- zwp(1);
    zwp(1)=[];
    aux=((x'*x)\(x'*zwp))';
    par_NN(l+1)=aux(2);  
    
    k=k+3;
    l=l+4;
    
end

estima{jj,1}=parsF;
estima{jj,2}=pars_beta;
estima{jj,3}=par_NN;

clearvars -except s sc wr beta jj options estima

%% Sample size: 1000
load eq3
 
jj=jj+1;

probability=[pi1(:,1);pi1(:,2)];
n=[10000 1000];

rng(390);

[prob, ~]=sim_crosssection(n,probability);

parsF=zeros(size(prob,1)*4,size(wr,2));
pars_beta=zeros(size(prob,1)*4,size(wr,2));
par_NN=zeros(size(prob,1)*4,1);

k=1;
l=1;

for i=1:size(prob,1) %Loop over the repetions 
    
    probs=[prob(i,1:4)' prob(i,5:8)'];
 
    for j=1:size(wr,2) %Loop over the scrap values
        
        try
           
           %Method B 
           zf=-[1;1;0;0];
           zw=[0;0;1;1;];
           z1=[probs(:,2) 1-probs(:,2)];
           x=[z1 zf];
           zwp=norminv(1-probs(:,1))-zw*wr(j);
           aux=((x'*x)\(x'*zwp))';
           parsF(l+1,j)=aux(3);  
           parsF(l,j) = lsqnonlin(@(dis) gmm_estimationF(probs,s,dis,wr(j),1,parsF(l+1,j)),.9,0,1-0.000001, options);
           parsF(l+2:l+3,j) = gmm_estimationF(probs,s,parsF(l,j),wr(j),0,parsF(l+1,j)); 
            
           %Method A 
           pars_beta(l,j)=lsqnonlin(@(dis) gmm_estimation(probs,s,dis,wr(j),1),.9,0,1-0.000001, options);
           pars_beta(l+1:l+3,j)=gmm_estimation(probs,s,pars_beta(l,j),wr(j),0);           


        catch
        end   
         

    end
    
    %N-N
    z1=[1-probs(:,2) probs(:,2)];
    z1(2:end,1)= z1(2:end,1)-z1(1,1);
    z1(2:end,2)= z1(2:end,2)-z1(1,2);
    z1(1,:)=[];
    z1=z1(:,1);
    x=[z1 sc];
    zwp=norminv(probs(:,1));
    zwp(2:end)=zwp(2:end)- zwp(1);
    zwp(1)=[];
    aux=((x'*x)\(x'*zwp))';
    par_NN(l+1)=aux(2);  
    
    k=k+3;
    l=l+4;
    
end
 
estima{jj,1}=parsF;
estima{jj,2}=pars_beta;
estima{jj,3}=par_NN;

clearvars -except s sc wr beta jj options estima

%% Sample size: 10000
load eq3
 
jj=jj+1;

probability=[pi1(:,1);pi1(:,2)];
n=[10000 10000];

rng(390);

[prob, ~]=sim_crosssection(n,probability);

parsF=zeros(size(prob,1)*4,size(wr,2));
pars_beta=zeros(size(prob,1)*4,size(wr,2));
par_NN=zeros(size(prob,1)*4,1);

k=1;
l=1;

for i=1:size(prob,1) %Loop over the repetions 
    
    probs=[prob(i,1:4)' prob(i,5:8)'];
 
    for j=1:size(wr,2) %Loop over the scrap values
        
        try
           
           %Method B 
           zf=-[1;1;0;0];
           zw=[0;0;1;1;];
           z1=[probs(:,2) 1-probs(:,2)];
           x=[z1 zf];
           zwp=norminv(1-probs(:,1))-zw*wr(j);
           aux=((x'*x)\(x'*zwp))';
           parsF(l+1,j)=aux(3);  
           parsF(l,j) = lsqnonlin(@(dis) gmm_estimationF(probs,s,dis,wr(j),1,parsF(l+1,j)),.9,0,1-0.000001, options);
           parsF(l+2:l+3,j) = gmm_estimationF(probs,s,parsF(l,j),wr(j),0,parsF(l+1,j)); 
            
           %Method A 
           pars_beta(l,j)=lsqnonlin(@(dis) gmm_estimation(probs,s,dis,wr(j),1),.9,0,1-0.000001, options);
           pars_beta(l+1:l+3,j)=gmm_estimation(probs,s,pars_beta(l,j),wr(j),0);           


        catch
        end   
         

    end
    
    %N-N
    z1=[1-probs(:,2) probs(:,2)];
    z1(2:end,1)= z1(2:end,1)-z1(1,1);
    z1(2:end,2)= z1(2:end,2)-z1(1,2);
    z1(1,:)=[];
    z1=z1(:,1);
    x=[z1 sc];
    zwp=norminv(probs(:,1));
    zwp(2:end)=zwp(2:end)- zwp(1);
    zwp(1)=[];
    aux=((x'*x)\(x'*zwp))';
    par_NN(l+1)=aux(2);  
    
    k=k+3;
    l=l+4;
    
end

estima{jj,1}=parsF;
estima{jj,2}=pars_beta;
estima{jj,3}=par_NN;

clearvars -except estima

%% Build the table

ra=zeros(30,3);
rb=zeros(30,3);
rc=zeros(30,1);

l=1;

tic
for i=1:3
    Ma=estima{i,2};
    Mb=estima{i,1};
    MNN=estima{i,3};
    
    vetaa=Ma(1:4:size(Ma,1),:);
    vetaa(isnan(sum(vetaa,2))==1,:)=[];
    vetaa(sum(vetaa,2)==0,:)=[];
    
    Fa=Ma(2:4:size(Ma,1),:);
    Fa(isnan(sum(Fa,2))==1,:)=[];
    Fa(sum(Fa,2)==0,:)=[];
    
    monopolya=Ma(3:4:size(Ma,1),:);
    monopolya(isnan(sum(monopolya,2))==1,:)=[];
    monopolya(sum(monopolya,2)==0,:)=[];
    
    oligopolya=Ma(4:4:size(Ma,1),:);
    oligopolya(isnan(sum(oligopolya,2))==1,:)=[];
    oligopolya(sum(oligopolya,2)==0,:)=[];
    
    somaa=Fa+repmat([0 .1 .2 ],size(Fa,1),1);
    
    ra(l:l+1,:)=[mean(vetaa);std(vetaa)];
    ra(l+2:l+3,:)=[mean(monopolya);std(monopolya)];
    ra(l+4:l+5,:)=[mean(oligopolya);std(oligopolya)];
    ra(l+6:l+7,:)=[mean(Fa);std(Fa)];
    ra(l+8:l+9,:)=[-mean(somaa);std(somaa)];
    
    %%%%%
    
    vetab=Mb(1:4:size(Mb,1),:);
    vetab(isnan(sum(vetab,2))==1,:)=[];
    vetab(sum(vetab,2)==0,:)=[];
    
    Fb=Mb(2:4:size(Mb,1),:);
    Fb(isnan(sum(Fb,2))==1,:)=[];
    Fb(sum(Fb,2)==0,:)=[];
    
    monopolyb=Mb(3:4:size(Mb,1),:);
    monopolyb(isnan(sum(monopolyb,2))==1,:)=[];
    monopolyb(sum(monopolyb,2)==0,:)=[];    
    
    oligopolyb=Mb(4:4:size(Mb,1),:);
    oligopolyb(isnan(sum(oligopolyb,2))==1,:)=[];
    oligopolyb(sum(oligopolyb,2)==0,:)=[];
    
    somab=Fb+repmat([0 .1 .2],size(Fb,1),1);
    
    rb(l:l+1,:)=[mean(vetab);std(vetab)];
    rb(l+2:l+3,:)=[mean(monopolyb);std(monopolyb)];
    rb(l+4:l+5,:)=[mean(oligopolyb);std(oligopolyb)];
    rb(l+6:l+7,:)=[mean(Fb);std(Fb)];
    rb(l+8:l+9,:)=[-mean(somab);std(somab)];
    
    %%%%
    
    FNN=MNN(2:4:size(Ma,1),:);
    FNN(isnan(FNN))=[];
    FNN(isinf(FNN))=[];
    FNN(sum(FNN,2)==0,:)=[];
    
    rc(l+8:l+9,:)=[mean(FNN);std(FNN)];
    
    l=l+10;
end
toc

%Table in term of the paremters values
t=[ra rb rc];

%Table in term of bias (as presented in the paper)
true_parameters=[.9 .9 .9;0 0 0;1.2 1.2 1.2;0 0 0;-1.2 -1.2 -1.2; 0 0 0;-.2 -.2 -.2;0 0 0;.1 .1 .1;0 0 0];
true_parameters=[repmat(true_parameters,3,2) repmat([0;0;0;0;0;0;0;0;.1;0],3,1)];
tb=t-true_parameters;

%Transforming the table into a string matrix
tbn=num2str(round(tb,4));
num=[['100   ';'      '; '      ';'      ';'      ';'      ';'      ';'      ';'      ';'      '];['1000  ';'      '; '      ';'      ';'      ';'      ';'      ';'      ';'      ';'      '];['10000 ';'      '; '      ';'      ';'      ';'      ';'      ';'      ';'      ';'      ']];
tbn=[num repmat(['Beta ';'     '; 'Mu1  ';'     ';'Mu2  ';'     ';'F    ';'     ';'Sunk ';'     '],3,1) tbn];
tbn=[['                         Method A              ' '              Method B              ' '   N-N ';'N     ' 'W    ' '     0     ' '    0.1    ' '    0.2    ' '       0   ' '       0.1 ' '       0.2 ' '           - '];tbn];

clearvars -except t tb tbn